@@ -134,7 +134,7 @@ Is nullable.
 =cut
 
 column alias =>
-  { data_type => "integer", is_foreign_key => 1, is_nullable => 1 };
+  { data_type => "integer", is_nullable => 1 };
 
 =head2 parent_id
 
@@ -226,6 +226,36 @@ sub new {
     return $new;
 }
 
+=head2 active_child_count
+
+See L<Interchange6::Schema::ResultSet::Navigation/with_active_child_count>
+for a resultset method which will prefill this data.
+
+=cut
+
+sub active_child_count {
+    my $self = shift;
+    if ( $self->has_column_loaded('active_child_count') ) {
+        return $self->get_column('active_child_count');
+    }
+    return $self->active_children->count;
+}
+
+=head2 active_product_count
+
+See L<Interchange6::Schema::ResultSet::Navigation/with_active_product_count>
+for a resultset method which will prefill this data.
+
+=cut
+
+sub active_product_count {
+    my $self = shift;
+    if ( $self->has_column_loaded('active_product_count') ) {
+        return $self->get_column('active_product_count');
+    }
+    return $self->products->active->count;
+}
+
 =head2 generate_uri($attrs)
 
 Called by L</new> if no uri is given as an argument.
@@ -278,6 +308,9 @@ sub generate_uri {
           unless utf8::is_utf8($uri);
     }
     catch {
+        # I don't know of a way to reach this catch so for now mark uncoverable
+        # uncoverable subroutine
+        # uncoverable statement
         $self->throw_exception(
             "Navigation->generate_uri failed to decode UTF-8 text: $_" );
     };
@@ -400,6 +433,20 @@ has_many
     };
   };
 
+=head2 media_navigations
+
+Type: has_many
+
+Related object: L<Interchange6::Schema::Result::MediaNavigation>
+
+=cut
+
+has_many
+  media_navigations => "Interchange6::Schema::Result::MediaNavigation",
+  "navigation_id",
+  { cascade_copy => 0, cascade_delete => 0 };
+
+
 =head2 navigation_products
 
 Type: has_many
