@@ -55,7 +55,6 @@ via L</user> relationship.
 
 column users_id => {
     data_type         => "integer",
-    is_foreign_key    => 1,
 };
 
 =head2 email
@@ -79,7 +78,6 @@ via L</shipping_address> relationship.
 
 column shipping_addresses_id => {
     data_type         => "integer",
-    is_foreign_key    => 1
 };
 
 =head2 billing_addresses_id
@@ -91,18 +89,17 @@ via L</billing_address> relationship.
 
 column billing_addresses_id => {
     data_type         => "integer",
-    is_foreign_key    => 1
 };
 
 =head2 weight
 
-Total numeric weight of the order. Default is 0.0.
+Total numeric weight of the order. Default is 0
 
 =cut
 
 column weight => {
     data_type         => "numeric",
-    default_value     => "0.0",
+    default_value     => 0,
     size              => [ 11, 3 ]
 };
 
@@ -158,60 +155,63 @@ column shipping_method => {
 
 The sum of all items in the cart before tax and shipping costs.
 
+Defaults to 0.
+
 =cut
 
 column subtotal => {
     data_type         => "numeric",
-    default_value     => "0.0",
-    size              => [ 11, 2 ],
+    default_value     => 0,
+    size              => [ 21, 3 ],
 };
 
 =head2 shipping
 
-The numeric cost associated with shipping the order. Default is 0.0.
+The numeric cost associated with shipping the order. Default is 0
 
 =cut
 
 column shipping => {
     data_type         => "numeric",
-    default_value     => "0.0",
-    size              => [ 11, 2 ],
+    default_value     => 0,
+    size              => [ 21, 3 ],
 };
 
 =head2 handling
 
-The numeric cost associated with handling the order. Default is 0.0.
+The numeric cost associated with handling the order. Default is 0.
 
 =cut
 
 column handling => {
     data_type         => "numeric",
-    default_value     => "0.0",
-    size              => [ 11, 2 ],
+    default_value     => 0,
+    size              => [ 21, 3 ],
 };
 
 =head2 salestax
 
-The total tax applied to the order. Default is 0.0.
+The total tax applied to the order. Default is 0
 
 =cut
 
 column salestax => {
     data_type         => "numeric",
-    default_value     => "0.0",
-    size              => [ 11, 2 ],
+    default_value     => 0,
+    size              => [ 21, 3 ],
 };
 
 =head2 total_cost
 
-The total cost of all items shipping handling and tax for the order. Default is 0.0.
+The total cost of all items shipping handling and tax for the order.
+Default is 0.
 
 =cut
 
 column total_cost => {
     data_type         => "numeric",
-    default_value     => "0.0",
-    size              => [ 11, 2 ],
+    default_value     => 0,
+    size              => [ 21, 3 ],
 };
 
 =head1 RELATIONS
@@ -397,7 +397,7 @@ sub set_comments {
         "set_comments needs a list of objects or hashrefs");
     my @to_set = ( ref( $_[0] ) eq 'ARRAY' ? @{ $_[0] } : @_ );
     $self->order_comments->delete_all;
-    $self->add_to_comments( $_, ref( $_[1] ) ? $_[1] : {} ) for (@to_set);
+    $self->add_to_comments( $_ ) for (@to_set);
 }
 
 =head2 delete
