@@ -14,9 +14,15 @@ use DateTime;
 use Encode;
 use Try::Tiny;
 
-use Interchange6::Schema::Candy -components =>
-  [qw(Helper::Row::OnColumnChange Helper::Row::ProxyResultSetMethod
-      Helper::Row::SelfResultSet InflateColumn::DateTime TimeStamp)];
+use Interchange6::Schema::Candy -components => [
+    qw(
+      InflateColumn::DateTime
+      TimeStamp
+      Helper::Row::SelfResultSet
+      Helper::Row::ProxyResultSetMethod
+      Helper::Row::OnColumnChange
+      )
+];
 
 =head1 DESCRIPTION
 
@@ -104,7 +110,9 @@ column description => {
 
 =head2 price
 
-Numeric value representing product cost. Default is 0.0.
+Numeric value representing product cost.
+
+Defaults to 0.
 
 When C<price> is updated and product has related
 L<Interchange6::Schema::Result::PriceModifier/discount> then also update
@@ -113,10 +121,20 @@ This is done using the method C<update_price_modifiers>.
 
 =cut
 
+# Max decimal places used by any currency as of 2015-12-01 is 3
+#
+# Note on amount of storage used by different backends for numeric/decimal:
+#
+# Pg: depends on the actual value being stored
+# MySQL: 4 bytes for every 9 digits before and after the decimal point
+#        with different amount for 'leftover' digits.
+#        See: http://dev.mysql.com/doc/refman/5.1/en/precision-math-decimal-characteristics.html
+#        So 20,3 takes 8 bytes for lhs and 2 for rhs = 10 total
+#
 column price => {
     data_type          => "numeric",
-    default_value      => "0.0",
-    size               => [ 10, 2 ],
+    size               => [ 21, 3 ],
+    default_value      => 0,
     keep_storage_value => 1,
 };
 
@@ -282,8 +300,7 @@ Related object: L<Interchange6::Schema::Result::PriceModifier>
 
 has_many
   price_modifiers => "Interchange6::Schema::Result::PriceModifier",
-  "sku",
-  { cascade_copy => 0, cascade_delete => 0 };
+  "sku";
 
 =head2 inventory
 
@@ -394,29 +411,27 @@ Type: many_to_many with media
 
 many_to_many media => "media_products", "media";
 
-=head2 _product_reviews
+=head2 product_messages
 
 Type: has_many
 
-Related object: L<Interchange6::Schema::Result::ProductReview>
-
-This is considered a private method. Please see public L</product_reviews> method.
+Related object: L<Interchange6::Schema::Result::ProductMessage>
 
 =cut
 
 has_many
-  _product_reviews => "Interchange6::Schema::Result::ProductReview",
+  product_messages => "Interchange6::Schema::Result::ProductMessage",
   "sku", { cascade_copy => 0 };
 
-=head2 _reviews
+=head2 messages
 
 Type: many_to_many
 
-This is considered a private method. Accessor to related Message results. Please see public L</reviews> and L</add_to_reviews> methods.
+Accessor to related Message results.
 
 =cut
 
-many_to_many _reviews => "_product_reviews", "message";
+many_to_many messages => "product_messages", "message";
 
 =head1 METHODS
 
@@ -431,9 +446,7 @@ and L</sku> have been supplied as arguments but L</uri> has not.
 
 sub insert {
     my ( $self, @args ) = @_;
-    if ( $self->name && $self->sku && !$self->uri ) {
-        $self->generate_uri;
-    }
+    $self->generate_uri unless $self->uri;
     $self->next::method(@args);
     return $self;
 }
@@ -513,6 +526,9 @@ sub generate_uri {
           unless utf8::is_utf8($uri);
     }
     catch {
+        # Haven't yet found a way to get here :)
+        # uncoverable subroutine
+        # uncoverable statement
         $self->throw_exception(
             "Product->generate_uri failed to decode UTF-8 text: $_" );
     };
@@ -581,94 +597,94 @@ sub path {
     return wantarray ? @path : \@path;
 }
 
-=head2 tier_pricing
-
-Tier pricing can be calculated for a single role and also a combination of several roles.
-
-=over 4
-
-=item Arguments: array reference of L<Role names|Interchange6::Schema::Result::Role/name>
-
-=item Return Value: in scalar context an array reference ordered by quantity ascending of hash references of quantity and price, in list context returns an array instead of array reference
-
-=back
-
-The method always returns the best price for specific price points including
-any PriceModifier rows where roles_id is undef.
-
-  my $aref = $product->tier_pricing( 'trade' );
-
-  # [ 
-  #   { quantity => 1,   price => 20 }, 
-  #   { quantity => 10,  price => 19 }, 
-  #   { quantity => 100, price => 18 }, 
-  # ]
-
-=cut
-
-# TODO: SysPete is not happy with the initial version of this method.
-# Patches always welcome.
-
-sub tier_pricing {
-    my ( $self, $args ) = @_;
-
-    my $cond = { 'role.name' => undef };
-
-    if ( $args ) {
-        $self->throw_exception(
-            "Argument to tier_pricing must be an array reference")
-          unless ref($args) eq 'ARRAY';
-
-        $cond = { 'role.name' => [ undef, { -in => $args } ] };
-    }
-
-    my @result = $self->price_modifiers->search(
-        $cond,
-        {
-            join   => 'role',
-            select => [ 'quantity', { min => 'price' } ],
-            as       => [ 'quantity', 'price' ],
-            group_by => 'quantity',
-            order_by => { -asc => 'quantity' },
-            result_class => 'DBIx::Class::ResultClass::HashRefInflator',
-        },
-    )->all;
-
-    if ( scalar @result && $result[0]->{quantity} < 1 ) {
-
-        # zero or minus qty should not be possible so we adjust to one if found
-
-        $result[0]->{quantity} = 1;
-    }
-
-    # maybe no qty 1 tier is defined so make sure we've got one
-
-    if ( scalar @result && $result[0]->{quantity} == 1 ) {
-        $result[0]->{price} = $self->price
-          if $self->price < $result[0]->{price};
-    }
-    else {
-        unshift @result, +{ quantity => 1, price => $self->price };
-    }
-
-    # Remove quantities that are inappropriate due to price at higher
-    # quantity being higher (or same as) that a price at a lower quantity.
-    # Normally caused when there are different price breaks for different
-    # roles but we have been asked to combine multiple roles.
-
-    my @return;
-    my $previous;
-    foreach my $i ( @result ) {
-        push @return, $i;
-        unless ( defined $previous ) {
-            $previous = $i->{price};
-            next;
-        }
-        pop @return unless $i->{price} < $previous;
-    }
-
-    return wantarray ? @return : \@return;
-}
+#=head2 tier_pricing
+#
+#Tier pricing can be calculated for a single role and also a combination of several roles.
+#
+#=over 4
+#
+#=item Arguments: array reference of L<Role names|Interchange6::Schema::Result::Role/name>
+#
+#=item Return Value: in scalar context an array reference ordered by quantity ascending of hash references of quantity and price, in list context returns an array instead of array reference
+#
+#=back
+#
+#The method always returns the best price for specific price points including
+#any PriceModifier rows where roles_id is undef.
+#
+#  my $aref = $product->tier_pricing( 'trade' );
+#
+#  # [ 
+#  #   { quantity => 1,   price => 20 }, 
+#  #   { quantity => 10,  price => 19 }, 
+#  #   { quantity => 100, price => 18 }, 
+#  # ]
+#
+#=cut
+#
+## TODO: SysPete is not happy with the initial version of this method.
+## Patches always welcome.
+#
+#sub tier_pricing {
+#    my ( $self, $args ) = @_;
+#
+#    my $cond = { 'role.name' => undef };
+#
+#    if ( $args ) {
+#        $self->throw_exception(
+#            "Argument to tier_pricing must be an array reference")
+#          unless ref($args) eq 'ARRAY';
+#
+#        $cond = { 'role.name' => [ undef, { -in => $args } ] };
+#    }
+#
+#    my @result = $self->price_modifiers->search(
+#        $cond,
+#        {
+#            join   => 'role',
+#            select => [ 'quantity', { min => 'price' } ],
+#            as       => [ 'quantity', 'price' ],
+#            group_by => 'quantity',
+#            order_by => { -asc => 'quantity' },
+#            result_class => 'DBIx::Class::ResultClass::HashRefInflator',
+#        },
+#    )->all;
+#
+#    if ( scalar @result && $result[0]->{quantity} < 1 ) {
+#
+#        # zero or minus qty should not be possible so we adjust to one if found
+#
+#        $result[0]->{quantity} = 1;
+#    }
+#
+#    # maybe no qty 1 tier is defined so make sure we've got one
+#
+#    if ( scalar @result && $result[0]->{quantity} == 1 ) {
+#        $result[0]->{price} = $self->price
+#          if $self->price < $result[0]->{price};
+#    }
+#    else {
+#        unshift @result, +{ quantity => 1, price => $self->price };
+#    }
+#
+#    # Remove quantities that are inappropriate due to price at higher
+#    # quantity being higher (or same as) that a price at a lower quantity.
+#    # Normally caused when there are different price breaks for different
+#    # roles but we have been asked to combine multiple roles.
+#
+#    my @return;
+#    my $previous;
+#    foreach my $i ( @result ) {
+#        push @return, $i;
+#        unless ( defined $previous ) {
+#            $previous = $i->{price};
+#            next;
+#        }
+#        pop @return unless $i->{price} < $previous;
+#    }
+#
+#    return wantarray ? @return : \@return;
+#}
 
 =head2 selling_price
 
@@ -678,17 +694,21 @@ Arguments should be given as a hash reference with the following keys/values:
 
 =item * quantity => $quantity
 
-=item * roles => array reference of L<Role names|Interchange6::Schema::Result::Role/name>
+C<quantity> defaults to 1 if not supplied.
 
 =back
 
-PriceModifier rows which have roles_is undefined are always included in the search in addition to any C<roles> that are provided as arguments. This enables promotional prices to be specified between fixed dates in L<Pricing price|Interchange6::Schema::Result::Pricing> to apply to all classes of user whether logged in or not.
+PriceModifier rows which have C<roles_id> undefined are always included in the
+search in addition to any C<roles> that belonging to L<Schema/logger_in_user>.
+This enables promotional prices to be specified between fixed dates in
+L<Interchange6::Schema::Result::PriceModifier/price> to apply to all classes
+of user whether logged in or not.
 
-C<quantity> defaults to 1 if not supplied.
+Returns lowest price from L</price> and
+L<Interchange6::Schema::Result::PriceModifier/price>.
 
-Returns lowest price from L</price> and L<Pricing price|Interchange6::Schema::Result::Pricing/price>.
-
-Throws exception on bad arguments though unexpected keys in the hash reference will be silently discarded.
+Throws exception on bad arguments though unexpected keys in the hash reference
+will be silently discarded.
 
 If the query was constructed using
 L<Interchange6::Schema::ResultSet::Product/with_lowest_selling_price> then
@@ -700,6 +720,8 @@ arguments are supplied in which case a new query is performed.
 sub selling_price {
     my ( $self, $args ) = @_;
 
+    my $schema = $self->result_source->schema;
+
     my $price = $self->price;
 
     if ( $self->has_column_loaded('selling_price') && !defined $args ) {
@@ -729,33 +751,36 @@ sub selling_price {
         $args->{quantity} = 1;
     }
 
-    # roles
+    # start building the the search condition
 
-    my $role_cond = undef;
+    my $today = $schema->format_datetime(DateTime->today);
 
-    if ( $args->{roles} ) {
-        $self->throw_exception(
-            "Argument roles to selling price must be an array reference")
-          unless ref( $args->{roles} ) eq 'ARRAY';
+    my $search_condition = {
+        quantity   => { '<=', $args->{quantity} },
+        start_date => [ undef, { '<=', $today } ],
+        end_date   => [ undef, { '>=', $today } ],
+        roles_id   => undef,
+    };
 
-        $role_cond = [ undef, { -in => $args->{roles} } ];
+    if ( my $user = $schema->current_user ) {
+
+        # add roles_id condition
+
+        $search_condition->{roles_id} = [
+            undef,
+            {
+                -in => $schema->resultset('UserRole')
+                  ->search( { users_id => $user->id } )->get_column('roles_id')
+                  ->as_query
+            }
+        ];
     }
 
     # now finally we can see if there is a better price for this customer
 
-    my $today = $self->result_source->schema->format_datetime(DateTime->today);
-
-    my $selling_price = $self->price_modifiers->search(
-        {
-            'role.name' => $role_cond,
-            quantity => { '<=', $args->{quantity} },
-            start_date => [ undef, { '<=', $today } ],
-            end_date   => [ undef, { '>=', $today } ],
-        },
-        {
-            join => 'role',
-        },
-    )->get_column('price')->min;
+    my $selling_price =
+      $self->price_modifiers->search($search_condition)->get_column('price')
+      ->min;
 
     return
       defined $selling_price
@@ -823,8 +848,8 @@ in case of failure).
 sub find_variant {
     my ( $self, $input, $match_info ) = @_;
 
-    if ( my $canonical = $self->canonical ) {
-        return $canonical->find_variant( $input, $match_info );
+    if ( $self->canonical_sku ) {
+        return $self->canonical->find_variant( $input, $match_info );
     }
 
     my $gather_matches;
@@ -1125,9 +1150,7 @@ sub add_variants {
     my $attr_rs = $self->result_source->schema->resultset('Attribute');
 
     for my $var_ref (@variants) {
-        my %attr;
-        my %product;
-        my $sku;
+        my ( %attr, %product, $sku );
 
         unless ( exists $var_ref->{sku} && ( $sku = $var_ref->{sku} ) ) {
             die "SKU missing in input for add_variants.";
@@ -1251,18 +1274,29 @@ sub media_by_type {
 
 =head2 product_reviews
 
-Reviews should only be associated with parent products. This method returns the related ProductReview records for a parent product. For a child product the ProductReview records for the parent are returned.
+Reviews should only be associated with parent products.
+
+This method returns the related L<Interchange6::Schema::Result::ProductMessage>
+records for a parent product where the related
+L<Interchange6::Schema::Result::Message> has
+L<Interchange6::Schema::Result::MessageType/name> of C<product_review>.
+For a child product the ProductReview records for the parent are returned.
 
 =cut
 
 sub product_reviews {
     my $self = shift;
-    if ( $self->canonical_sku ) {
-        return $self->canonical->_product_reviews;
-    }
-    else {
-        return $self->_product_reviews;
-    }
+
+    $self = $self->canonical if $self->canonical_sku;
+
+    return $self->product_messages->search(
+        {
+            'message_type.name' => 'product_review',
+        },
+        {
+            join => { message => 'message_type' },
+        }
+    );
 }
 
 =head2 reviews
@@ -1285,7 +1319,7 @@ sub reviews {
     # use parent if I have one
     $self = $self->canonical if $self->canonical_sku;
 
-    return $self->_reviews->search(@_);
+    return $self->product_reviews->search_related('message', @_);
 }
 
 =head2 top_reviews
@@ -1371,12 +1405,14 @@ sub add_to_reviews {
             }
         }
     }
-    else {
-        push @_, type => "product_review";
-        $obj = $rset_message->create( {@_} );
-    }
-    my $sku = $self->canonical_sku || $self->sku;
-    $self->product_reviews->create( { sku => $sku, messages_id => $obj->id } );
+
+    $self->throw_exception("Bad argument supplied to add_to_reviews")
+      unless $obj;
+
+    # uncoverable condition left
+    # uncoverable condition false
+    my $sku = $self->canonical_sku ? $self->canonical_sku : $self->sku;
+    $self->product_messages->create( { sku => $sku, messages_id => $obj->id } );
     return $obj;
 }
 
@@ -1401,7 +1437,7 @@ sub set_reviews {
         "set_reviews needs a list of objects or hashrefs" );
     my @to_set = ( ref( $_[0] ) eq 'ARRAY' ? @{ $_[0] } : @_ );
     $self->product_reviews->delete_all;
-    $self->add_to_reviews( $_, ref( $_[1] ) ? $_[1] : {} ) for (@to_set);
+    $self->add_to_reviews( $_ ) for (@to_set);
 }
 
 =head2 quantity_in_stock
