@@ -9,7 +9,7 @@ test 'base attribute tests' => sub {
 
     my $self = shift;
 
-    my ( $count, %navigation, $product, %size, $meta, $ret, $rset );
+    my ( $count, %navigation, $product, %size, $meta, $ret, @ret, $rset );
 
     my $schema = $self->ic6s_schema;
 
@@ -156,14 +156,14 @@ qr/find_attribute_value input requires at least a valid attribute value/,
     cmp_ok( $attr_name, 'eq', 'bar_flavor',
         "Testing name of first attribute returned" );
 
-    # return a list of all product attributes and attribute_values
+    # return an arrayref of all product attributes and attribute_values
     lives_ok(
         sub {
             $ret = $product->search_attribute_values(
                 undef, { order_by => 'priority' }, { order_by => 'priority' }
                 );
             },
-        "Create attribute and attribute_value list"
+        "Create attribute and attribute_value arrayref"
     );
 
     cmp_deeply(
@@ -214,6 +214,66 @@ qr/find_attribute_value input requires at least a valid attribute value/,
         "Deep comparison is good"
         ) or diag Dumper($ret);
 
+    # return an array of all product attributes and attribute_values
+    lives_ok(
+        sub {
+            @ret = $product->search_attribute_values(
+                undef,
+                { order_by => 'priority' },
+                { order_by => 'priority' }
+            );
+        },
+        "Create attribute and attribute_value array"
+    );
+
+    cmp_deeply(
+        \@ret,
+        bag(
+            {
+                'priority'         => 0,
+                'attribute_values' => [
+                    {
+                        'priority'            => 1,
+                        'attributes_id'       => re(qr/^\d+$/),
+                        'value'               => 'vanilla',
+                        'attribute_values_id' => re(qr/^\d+$/),
+                        'title'               => 'Vanilla'
+                    },
+                    {
+                        'priority'            => 2,
+                        'attributes_id'       => re(qr/^\d+$/),
+                        'value'               => 'mint',
+                        'attribute_values_id' => re(qr/^\d+$/),
+                        'title'               => 'Mint'
+                    }
+                ],
+                'attributes_id' => re(qr/^\d+$/),
+                'dynamic'       => 0,
+                'name'          => 'bar_flavor',
+                'title'         => 'Choose Flavor',
+                'type'          => 'menu'
+            },
+            {
+                'priority'         => 0,
+                'attribute_values' => [
+                    {
+                        'priority'            => 1,
+                        'attributes_id'       => re(qr/^\d+$/),
+                        'value'               => 'small',
+                        'attribute_values_id' => re(qr/^\d+$/),
+                        'title'               => 'Small'
+                    }
+                ],
+                'attributes_id' => re(qr/^\d+$/),
+                'dynamic'       => 0,
+                'name'          => 'bar_size',
+                'title'         => 'Choose Size',
+                'type'          => 'menu'
+            }
+        ),
+        "Deep comparison is good"
+    ) or diag Dumper($ret);
+
     lives_ok(
         sub {
             $navigation{bananas} =
