@@ -33,6 +33,16 @@ test 'simple user tests' => sub {
         "fail User create with undef username"
     );
 
+    lives_ok(
+        sub {
+            $result =
+              $rset_user->create(
+                { username => "AnonymousUsername", is_anonymous => 1 } );
+        },
+        "User create with good username and is_anonymous => 1"
+    );
+    $user_count++;
+
     lives_ok(
         sub {
             $result =
@@ -59,6 +69,11 @@ test 'simple user tests' => sub {
 
     cmp_ok( $roles->first->name, 'eq', "anonymous", "role is anonymous" );
 
+    # code coverage
+    lives_ok { $result = $rset_user->find( { users_id => $result->id } ) }
+    "find user using { users_id => \$id }";
+    ok $result, "got the user";
+
     lives_ok(
         sub {
             $result =
@@ -90,6 +105,7 @@ test 'simple user tests' => sub {
         },
         "create user"
     );
+    cmp_ok $result->username, 'eq', 'nevairbe@nitesi.de', "username is correct";
 
     lives_ok( sub { $roles = $result->roles }, "get roles" );
 
@@ -97,6 +113,12 @@ test 'simple user tests' => sub {
 
     cmp_ok( $roles->first->name, 'eq', "user", "role is user" );
 
+    dies_ok { $result->username(undef) } "cannot change username to undef";
+
+    dies_ok { $result->username('') } "cannot change username to empty string";
+
+    lives_ok { $result->username('nevairbe@nitesi.de') } "change username";
+
     throws_ok(
         sub { $rset_user->create( { username => 'nevairbe@nitesi.de' } ) },
         qr/DBI Exception/i,
@@ -166,6 +188,11 @@ test 'simple user tests' => sub {
     ok(!$result->check_password(''), "cannot login with empty password");
     ok(!$result->check_password(undef), "cannot login with undef password");
 
+    lives_ok { $result = $self->users->find({ nickname => "Cust1" }) }
+    "find user by nickname";
+
+    cmp_ok $result->username, 'eq', 'customer1', 'we found customer1';
+
     $self->clear_users;
 };
 
@@ -329,11 +356,16 @@ test 'password reset' => sub {
     my $self   = shift;
     my $schema = $self->ic6s_schema;
 
-    my ( $user, $token, $dt );
+    my ( @users, $user, $token, $dt, $result );
 
     # make sure our test user starts off nice and clean
+    lives_ok {
+        @users = $self->users->search( undef, { order_by => 'username' } )->all
+    }
+    "Get an array of all users";
 
-    lives_ok( sub { $user = $self->users->first }, "get a user" );
+    lives_ok { $user = $users[0] } "get first user from array";
+    ok $user, "We have a user";
 
     lives_ok( sub { $user->reset_expires(undef) },
         "set reset_expires to undef" );
@@ -351,9 +383,11 @@ test 'password reset' => sub {
 
     like( $token, qr/^\w{22}_\w{32}$/, "token with checksum looks good" );
 
+    # stash DB reset_token field for later use
     $token =~ m/^(\w+)_/;
+    my $db_token = $1;
 
-    cmp_ok( $1, 'eq', $user->reset_token,
+    cmp_ok( $db_token, 'eq', $user->reset_token,
         "token matches reset_token in db" );
 
     $dt = DateTime->now->add( hours => 23 );
@@ -372,9 +406,21 @@ test 'password reset' => sub {
 
     ok(
         !$user->reset_token_verify($token),
-        "old token with checksum no longer valud"
+        "old token with checksum no longer valid"
     );
 
+    lives_ok { $users[1]->update({ reset_token => $db_token }) }
+    "give old reset token to some other user";
+
+    ok !$users[1]->reset_token_verify($token),
+      "verify token against other user fails";
+
+    my $user2;
+    lives_ok { $user2 = $self->users->find_user_with_reset_token($token) }
+    "find_user_with_reset_token should not find a user";
+
+    ok !$user2, "yup - no user found";
+
     # test failure on changed password
 
     lives_ok( sub { $token = $user->reset_token_generate }, "get reset token" );
@@ -491,6 +537,20 @@ test 'password reset' => sub {
 
     # find_user_with_reset_token resultset method
 
+    lives_ok( sub { set_relative_time(-600) }, "put clock back 10 minutes" );
+
+    lives_ok { $result = $self->users->find_user_with_reset_token( $token ) }
+    "find_user_with_reset_token lives";
+
+    cmp_ok $user->id, '==', $result->id, "correct user returned";
+
+    lives_ok( sub { restore_time() }, "restore time" );
+
+    lives_ok { $result = $self->users->find_user_with_reset_token( $token ) }
+    "find_user_with_reset_token lives";
+
+    ok !defined $result, "token has expired" or diag explain $result;
+
     my $users_id = $user->id;
     my $user_count = $self->users->count;
 
@@ -508,7 +568,19 @@ test 'password reset' => sub {
     }
 
     cmp_ok( $self->users->search( { reset_token => { '!=' => undef } } )->count,
-        '==', $user_count, "add users now have a reset_token" );
+        '==', $user_count, "all users now have a reset_token" );
+
+    throws_ok { $user = $self->users->find_user_with_reset_token("q") }
+    qr/Bad argument to find_user_with_reset_token/,
+      "find_user_with_reset_token with bad arg 'q'";
+
+    throws_ok { $user = $self->users->find_user_with_reset_token("q_") }
+    qr/Bad argument to find_user_with_reset_token/,
+      "find_user_with_reset_token with bad arg 'q_'";
+
+    throws_ok { $user = $self->users->find_user_with_reset_token("_q") }
+    qr/Bad argument to find_user_with_reset_token/,
+      "find_user_with_reset_token with bad arg '_q'";
 
     lives_ok( sub { $user = $self->users->find_user_with_reset_token("q_q") },
         "find_user_with_reset_token with bad token" );
@@ -522,6 +594,14 @@ test 'password reset' => sub {
 
     cmp_ok( $user->users_id, '==', $users_id, "we got the right user" );
 
+    throws_ok( sub { $self->users->find_user_with_reset_token("_") },
+        qr/Bad argument to find_user_with_reset_token/,
+        "find_user_with_reset_token('_') dies" );
+
+    throws_ok( sub { $self->users->find_user_with_reset_token(undef) },
+        qr/Bad argument to find_user_with_reset_token/,
+        "find_user_with_reset_token(undef) dies" );
+
     # cleanup
     $self->clear_users;
 };
